//////////////////////////////////////////////////////////////////////
//
// File:     DocToken.java
// Project:  Java-based html tag extractor
// Author:   Scott Sanner, Stanford University
// Date:     4/12/2000
//
//////////////////////////////////////////////////////////////////////

// Package definition
package Extractor;

import java.io.*;
import java.util.*;
import java.lang.*;

// Main class - Represents a document token
public class DocToken
{
	/** Constants **/

	// Token type Constants
	public final static int _OTHER = 0;
	public final static int _WORD  = 1;
	public final static int _PUNCT = 2;
	public final static int _NUM   = 3;
	public final static int _TAG   = 4;
	public final static int _SDOC  = 5;
	public final static int _EDOC  = 6;

	// Token classifications
	public final static int _START      = 0;
	public final static int _BKGD       = 1;
	public final static int _PREFIX     = 2;
	public final static int _SUFFIX     = 3;
	public final static int _TARGET     = 4;
	public final static int _END        = 5;
	public final static int _UNKNOWN    = 6;
	public final static int _BOTH       = 7;
	public final static int _NUM_STATES = 8;
	public final static int _NUM_VALID_STATES = 6;

	/** Static Members **/
	public static SuffStatistics _ssStat = null;

	/** Members **/
	public String _sWord;
	public int    _nType;
	public int    _nClass;
	public int    _nObsIndex;

	public String _sExtra; // What was actually parsed
	public int    _nExtra; // If a number was parsed

	/** Methods **/
	public DocToken(String word)
	{
		this(word, _OTHER, _UNKNOWN, null);
	}

	public DocToken(String word, int type)
	{
		this(word, type, _UNKNOWN, null);
	}

	public DocToken(String word, int type, int nclass)
	{
		this(word, type, nclass, null);
	}

	public DocToken(String word, int type, int nclass, String sextra)
	{
		_sWord     = word;
		_nType     = type;
		_nClass    = nclass;
		_sExtra    = sextra;

		if (_ssStat != null)
			_nObsIndex =_ssStat.getIndex(word);
		else
			_nObsIndex = (-1);

		try {
			if (_nType == _NUM)
				_nExtra = Integer.parseInt(_sExtra);
			else
				_nExtra = 0;
		} catch (NumberFormatException e)
		{
			System.out.println("Tokenizer parsed incorrect" +
							   " integer... halting...");
			System.exit(1);
		}
	}

	public static void setSuffStats(SuffStatistics s)
	{
		_ssStat = s;
	}

	public void setNumber(int n)
	{
		_nExtra = n;
	}

	public String toString()
	{
		return getClassificationName(_nClass) + ":" + _nObsIndex +
			": '" +_sWord+/*"/"+_sExtra+*/"'" + 
			((_nType == _NUM) ? ("  (" + _sExtra + ")") : ("")); 
	}

	public static String getClassificationName(int c)
	{
		switch (c)
		{
		case _UNKNOWN: return "UNKNOWN";
		case _BKGD   : return "BKGD   ";
		case _START  : return "START  ";
		case _END    : return "END    ";
		case _PREFIX : return "PREFIX ";
		case _SUFFIX : return "SUFFIX ";
		case _BOTH   : return "BOTH   ";
		case _TARGET : return "TARGET ";
		default      : return "ILLEGAL";
		}
	}
}
