//////////////////////////////////////////////////////////////////////
//
// File:     HtmlExtractor.java
// Project:  Java-based html tag extractor
// Author:   Scott Sanner, Stanford University
// Date:     4/12/2000
//
//////////////////////////////////////////////////////////////////////

// Package definition
package Extractor;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;
import com.quiotix.html.parser.*;

// Main class - Extracts a token stream from an html document either web
//              or file based
public class HtmlExtractor
{
	/** Constants **/

	/** Member Fields **/
	
	/** Member Methods **/
	public HtmlExtractor()
	{
		// Nothing to initialize for now...
	}

	public boolean extract(String resname, HtmlVisitor hv) 
	{
		InputStream r;

		try {
			if (resname.startsWith("http"))
				{
					URL url = new URL(resname);
					r = url.openStream();
				}
			else
				{
					// Just assume it is a regular filename
					r = new FileInputStream(resname);
				}
		} catch (FileNotFoundException e) {
			System.err.println("\nFile I/O Error: Could not open '" + 
							   resname + "'\n");
			return false;
		} catch (MalformedURLException e) {
			System.err.println("\nHttp Error: Could not open '" + 
							   resname + "'\n");
			return false;
		} catch (IOException e) {
			System.err.println("\nIO Error: Could not open '" + 
							   resname + "'\n");
			return false;
		}

		try {
			HtmlDocument document;
			
			document = new HtmlParser(r).HtmlDocument();
			document.accept(new com.quiotix.html.parser.HtmlCollector());
			document.accept(new HtmlScrubber(HtmlScrubber.DEFAULT_OPTIONS 
											 | HtmlScrubber.TRIM_SPACES));
			document.accept(hv);

			r.close();
		}
		catch (Exception e) {
			System.err.println("Error: " + e);
			//e.printStackTrace();
			return false;
		}
		finally {
			return true;
		};
	}

	public static void main(String[] args)
	{
		if (args.length < 1)
			{
				System.out.println("\nError: Must pass filename\n\n");
				System.exit(1);
			}

		HtmlExtractor ex = new HtmlExtractor();
		if (!ex.extract(args[0], new HtmlWriter(System.out)))
			{
				System.err.println("\nError: Parsing aborted\n");
			}
	}
}


