//////////////////////////////////////////////////////////////////////
//
// File:     TextTokenizer.java
// Project:  Java-based html tag extractor
// Author:   Scott Sanner, Stanford University
// Date:     5/25/2000
//
//////////////////////////////////////////////////////////////////////

// Package definition
package Extractor;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

// Main class - Represents a set of tokenized html documents
public class TextTokenizer
{
	/** Constants **/
	public final static int _WHITESPACE  = 0;
	public final static int _LETTER      = 1;
	public final static int _DIGIT       = 2;
	public final static int _PUNCTUATION = 3;
	public final static int _UNKNOWN     = 4;

	/** Members **/
	public String _s;
	public String _cs;
	public int    _len;
	public int    _pos;

	/** Methods **/
	public TextTokenizer(String s)
	{
		_s   = s;
		_cs  = _s.toUpperCase();
		_len = _cs.length();
		_pos = 0;
	}

	public DocToken nextToken(int nclass)
	{
		char c;
		int toktype = DocToken._OTHER;
		StringBuffer sb = new StringBuffer();
		StringBuffer ab = new StringBuffer();
		int chartype = _UNKNOWN;
		int prev     = _UNKNOWN;
		boolean breakloop = false;
		boolean breakloopnow = false;
		
		// Filter initial whitespace
		while ((_pos < _len) && 
			   (classChar(_cs.charAt(_pos)) == _WHITESPACE))
			_pos++;
		
		// Read characters
		for (; _pos<_len && !breakloop; _pos++)
		{
			c = _cs.charAt(_pos);
			prev = chartype;
			chartype = classChar(c);
			switch (chartype)
			{
			case _WHITESPACE: 
			{ 
				breakloop = true; 
			} break;
			case _LETTER:     
			{ 
				if ((prev != _LETTER) && (prev != _UNKNOWN))
				{
					breakloopnow = true;
					break;
				}   
				sb.append(c);
				ab.append(_s.charAt(_pos));
				toktype = DocToken._WORD;
			} break;
			case _DIGIT:      
			{
				if ((prev != _DIGIT) && (prev != _UNKNOWN))
				{
					breakloopnow = true;
					break;
				}   
				if (prev == _UNKNOWN) sb.append("[NUMBER]");    
				ab.append(c);    
				toktype = DocToken._NUM;
            } break; 
			case _PUNCTUATION:      
			{
				if (prev != _UNKNOWN)
				{
					breakloopnow = true;
					break;
				}
				sb.append(c);
				ab.append(c);
				breakloop = true;
				toktype = DocToken._PUNCT;
            } break; 
			}

			if (breakloopnow) break;
		}

		// If valid string, return
		if (sb.length() > 0)
			return new DocToken(sb.toString(), toktype, 
								nclass, ab.toString());
		else
			return null;
	}

	public int classChar(char c)
	{
		if (Character.isLetter(c))
			return _LETTER;
		else if (Character.isWhitespace(c))
			return _WHITESPACE;
		else if (Character.isDigit(c))
			return _DIGIT;
		else 
			return _PUNCTUATION;
	}
}
